@@ -7,15 +7,15 @@
 from __future__ import annotations
 
 import logging
+import multiprocessing
 import os
 import signal
 import sys
 from contextlib import suppress
-from multiprocessing import set_start_method
 
 from ocrmypdf import __version__
+from ocrmypdf._pipelines.ocr import run_pipeline_cli
 from ocrmypdf._plugin_manager import get_parser_options_plugins
-from ocrmypdf._sync import run_pipeline
 from ocrmypdf._validation import check_options
 from ocrmypdf.api import Verbosity, configure_logging
 from ocrmypdf.exceptions import (
@@ -29,10 +29,16 @@ log = logging.getLogger('ocrmypdf')
 
 
 def sigbus(*args):
+    """Handle SIGBUS signals.
+
+    pikepdf, depending on configuration, may use mmap so SIGBUS is a
+    possibility.
+    """
     raise InputFileError("Lost access to the input file")
 
 
 def run(args=None):
+    """Run the ocrmypdf command line interface."""
     _parser, options, plugin_manager = get_parser_options_plugins(args=args)
 
     with suppress(AttributeError, PermissionError):
@@ -66,11 +72,12 @@ def run(args=None):
     with suppress(AttributeError, OSError):
         signal.signal(signal.SIGBUS, sigbus)
 
-    result = run_pipeline(options=options, plugin_manager=plugin_manager)
+    result = run_pipeline_cli(options=options, plugin_manager=plugin_manager)
     return result
 
 
 if __name__ == '__main__':
-    if sys.platform == 'darwin' and sys.version_info < (3, 8):
-        set_start_method('spawn')  # see python bpo-33725
+    multiprocessing.freeze_support()
+    if os.name == 'posix':
+        multiprocessing.set_start_method('forkserver')
     sys.exit(run())
