@@ -12,41 +12,52 @@ import re
 import sys
 from collections.abc import Iterable, Iterator, Sequence
 from contextlib import suppress
-from datetime import datetime, timezone
+from io import BytesIO
 from pathlib import Path
 from shutil import copyfileobj
 from typing import Any, BinaryIO, TypeVar, cast
 
 import img2pdf
 import pikepdf
-from pikepdf.models.metadata import encode_pdf_date
 from PIL import Image, ImageColor, ImageDraw
 
 from ocrmypdf._concurrent import Executor
 from ocrmypdf._exec import unpaper
 from ocrmypdf._jobcontext import PageContext, PdfContext
-from ocrmypdf._version import PROGRAM_NAME
-from ocrmypdf._version import __version__ as VERSION
+from ocrmypdf._metadata import repair_docinfo_nuls
 from ocrmypdf.exceptions import (
     DigitalSignatureError,
     DpiError,
     EncryptedPdfError,
     InputFileError,
     PriorOcrFoundError,
+    TaggedPDFError,
     UnsupportedImageFormatError,
 )
 from ocrmypdf.helpers import IMG2PDF_KWARGS, Resolution, safe_symlink
-from ocrmypdf.hocrtransform import HocrTransform
+from ocrmypdf.hocrtransform import DebugRenderOptions, HocrTransform
+from ocrmypdf.hocrtransform._font import Courier
 from ocrmypdf.pdfa import generate_pdfa_ps
 from ocrmypdf.pdfinfo import Colorspace, Encoding, PageInfo, PdfInfo
 from ocrmypdf.pluginspec import OrientationConfidence
 
+try:
+    from pi_heif import register_heif_opener
+except ImportError:
+
+    def register_heif_opener():
+        pass
+
+
 T = TypeVar("T")
 log = logging.getLogger(__name__)
 
 VECTOR_PAGE_DPI = 400
 
 
+register_heif_opener()
+
+
 def triage_image_file(input_file: Path, output_file: Path, options) -> None:
     """Triage the input image file.
 
@@ -131,7 +142,7 @@ def _pdf_guess_version(input_file: Path, search_window=1024) -> str:
     """
     with open(input_file, 'rb') as f:
         signature = f.read(search_window)
-    m = re.search(br'%PDF-(\d\.\d)', signature)
+    m = re.search(rb'%PDF-(\d\.\d)', signature)
     if m:
         return m.group(1).decode('ascii')
     return ''
@@ -148,8 +159,13 @@ def triage(
                     "Argument --image-dpi is being ignored because the "
                     "input file is a PDF, not an image."
                 )
-            # Origin file is a pdf create a symlink with pdf extension
-            safe_symlink(input_file, output_file)
+            try:
+                with pikepdf.open(input_file) as pdf:
+                    pdf.save(output_file)
+            except pikepdf.PdfError as e:
+                raise InputFileError() from e
+            except pikepdf.PasswordError as e:
+                raise EncryptedPdfError() from e
             return output_file
     except OSError as e:
         log.debug(f"Temporary file was at: {input_file}")
@@ -167,6 +183,7 @@ def get_pdfinfo(
     detailed_analysis: bool = False,
     progbar: bool = False,
     max_workers: int | None = None,
+    use_threads: bool = True,
     check_pages=None,
 ) -> PdfInfo:
     """Get the PDF info."""
@@ -176,6 +193,7 @@ def get_pdfinfo(
             detailed_analysis=detailed_analysis,
             progbar=progbar,
             max_workers=max_workers,
+            use_threads=use_threads,
             check_pages=check_pages,
             executor=executor,
         )
@@ -218,6 +236,16 @@ def validate_pdfinfo_options(context: PdfContext) -> None:
                     "form and all filled form fields. The output PDF will be "
                     "'flattened' and will no longer be fillable."
                 )
+    if pdfinfo.is_tagged:
+        if options.force_ocr or options.skip_text or options.redo_ocr:
+            log.warning(
+                "This PDF is marked as a Tagged PDF. This often indicates "
+                "that the PDF was generated from an office document and does "
+                "not need OCR. PDF pages processed by OCRmyPDF may not be "
+                "tagged correctly."
+            )
+        else:
+            raise TaggedPDFError()
     context.plugin_manager.hook.validate(pdfinfo=pdfinfo, options=options)
 
 
@@ -452,7 +480,7 @@ def calculate_raster_dpi(page_context: PageContext):
     page_dpi = get_page_square_dpi(page_context, image_dpi)
     if dpi_profile and dpi_profile.average_to_max_dpi_ratio < 0.8:
         log.warning(
-            "Weight average image DPI is %0.1f, max DPI is %0.1f. "
+            "Weighted average image DPI is %0.1f, max DPI is %0.1f. "
             "The discrepancy may indicate a high detail region on this page, "
             "but could also indicate a problem with the input PDF file. "
             "Page image will be rendered at %0.1f DPI.",
@@ -508,6 +536,7 @@ def rasterize(
                 device_idx = at_least('png16m')
 
     if pageinfo.has_vector:
+        log.debug("Page has vector content, using png16m")
         device_idx = at_least('png16m')
 
     device = colorspaces[device_idx]
@@ -644,7 +673,7 @@ def ocr_engine_hocr(input_file: Path, page_context: PageContext) -> tuple[Path,
         output_text=hocr_text_out,
         options=options,
     )
-    return (hocr_out, hocr_text_out)
+    return hocr_out, hocr_text_out
 
 
 def should_visible_page_image_use_jpg(pageinfo: PageInfo) -> bool:
@@ -701,19 +730,29 @@ def create_pdf_page_from_image(
     pageinfo = page_context.pageinfo
     pagesize = 72.0 * float(pageinfo.width_inches), 72.0 * float(pageinfo.height_inches)
     effective_rotation = (pageinfo.rotation - orientation_correction) % 360
-    if effective_rotation % 180 == 90:
+    swap_axis = effective_rotation % 180 == 90
+    if swap_axis:
         pagesize = pagesize[1], pagesize[0]
 
-    # This create a single page PDF
-    with open(image, 'rb') as imfile, open(output_file, 'wb') as pdf:
+    # Create a new single page PDF to hold
+    bio = BytesIO()
+    with open(image, 'rb') as imfile:
         log.debug('convert')
 
         layout_fun = img2pdf.get_layout_fun(pagesize)
         img2pdf.convert(
-            imfile, layout_fun=layout_fun, outputstream=pdf, **IMG2PDF_KWARGS
+            imfile,
+            layout_fun=layout_fun,
+            outputstream=bio,
+            engine=img2pdf.Engine.pikepdf,
+            rotation=img2pdf.Rotation.ifvalid,
         )
         log.debug('convert done')
 
+    # img2pdf does not generate boxes correctly, so we fix them
+    bio.seek(0)
+    fix_pagepdf_boxes(bio, output_file, page_context, swap_axis=swap_axis)
+
     output_file = page_context.plugin_manager.hook.filter_pdf_page(
         page=page_context, image_filename=image, output_pdf=output_file
     )
@@ -724,16 +763,33 @@ def render_hocr_page(hocr: Path, page_context: PageContext) -> Path:
     """Render the hOCR page to a PDF."""
     options = page_context.options
     output_file = page_context.get_path('ocr_hocr.pdf')
-    dpi = get_page_square_dpi(page_context, calculate_image_dpi(page_context))
-    debug_mode = options.pdf_renderer == 'hocrdebug'
+    if hocr.stat().st_size == 0:
+        # If hOCR file is empty (skipped page marker), create an empty PDF file
+        output_file.touch()
+        return output_file
 
-    hocrtransform = HocrTransform(hocr_filename=hocr, dpi=dpi.x)  # square
-    hocrtransform.to_pdf(
+    dpi = get_page_square_dpi(page_context, calculate_image_dpi(page_context))
+    debug_kwargs = {}
+    if options.pdf_renderer == 'hocrdebug':
+        debug_kwargs = dict(
+            debug_render_options=DebugRenderOptions(
+                render_baseline=True,
+                render_triangle=True,
+                render_line_bbox=False,
+                render_word_bbox=True,
+                render_paragraph_bbox=False,
+                render_space_bbox=False,
+            ),
+            font=Courier(),
+        )
+    HocrTransform(
+        hocr_filename=hocr,
+        dpi=dpi.to_scalar(),
+        **debug_kwargs,  # square
+    ).to_pdf(
         out_filename=output_file,
         image_filename=None,
-        show_bounding_boxes=False if not debug_mode else True,
-        invisible_text=True if not debug_mode else False,
-        interword_spaces=True,
+        invisible_text=True if not debug_kwargs else False,
     )
     return output_file
 
@@ -753,39 +809,57 @@ def ocr_engine_textonly_pdf(
         output_text=output_text,
         options=options,
     )
-    return (output_pdf, output_text)
+    return output_pdf, output_text
 
 
-def get_docinfo(base_pdf: pikepdf.Pdf, context: PdfContext) -> dict[str, str]:
-    """Read the document info and store it in a dictionary."""
-    options = context.options
+def _offset_rect(rect: tuple[float, float, float, float], offset: tuple[float, float]):
+    """Offset a rectangle by a given amount."""
+    return (
+        rect[0] + offset[0],
+        rect[1] + offset[1],
+        rect[2] + offset[0],
+        rect[3] + offset[1],
+    )
 
-    def from_document_info(key):
-        try:
-            s = base_pdf.docinfo[key]
-            return str(s)
-        except (KeyError, TypeError):
-            return ''
 
-    pdfmark = {
-        k: from_document_info(k)
-        for k in ('/Title', '/Author', '/Keywords', '/Subject', '/CreationDate')
-    }
-    if options.title:
-        pdfmark['/Title'] = options.title
-    if options.author:
-        pdfmark['/Author'] = options.author
-    if options.keywords:
-        pdfmark['/Keywords'] = options.keywords
-    if options.subject:
-        pdfmark['/Subject'] = options.subject
+def fix_pagepdf_boxes(
+    infile: Path | BinaryIO,
+    out_file: Path,
+    page_context: PageContext,
+    swap_axis: bool = False,
+) -> Path:
+    """Fix the bounding boxes in a single page PDF.
 
-    creator_tag = context.plugin_manager.hook.get_ocr_engine().creator_tag(options)
+    The single page PDF is created with a normal MediaBox with its lower left corner
+    at (0, 0). infile is the single page PDF. page_context.mediabox has the original
+    file's mediabox, which may have a different origin. We needto adjust the other
+    boxes in the single page PDF to match the effect they had on the original page.
 
-    pdfmark['/Creator'] = f'{PROGRAM_NAME} {VERSION} / {creator_tag}'
-    pdfmark['/Producer'] = f'pikepdf {pikepdf.__version__}'
-    pdfmark['/ModDate'] = encode_pdf_date(datetime.now(timezone.utc))
-    return pdfmark
+    When correcting page rotation, we create a single page PDF that is correctly
+    rotated instead of an incorrectly rotated and then setting page.Rotate on it.
+    If rotation is either 90 or 270 degrees, then this function can be called
+    with swap_axis to swap the X and Y coordinates of all the boxes.
+
+    We are not concerned with solving degenerate cases where the boxes overlap or
+    or express invalid rectangles. We merely pass the boxes, producing a
+    transformation equivalent to the change made by constructing a new page image.
+    """
+    with pikepdf.open(infile) as pdf:
+        for page in pdf.pages:
+            # page.BleedBox = page_context.pageinfo.bleedbox
+            # page.ArtBox = page_context.pageinfo.artbox
+            mediabox = page_context.pageinfo.mediabox
+            offset = mediabox[0], mediabox[1]
+            cropbox = _offset_rect(page_context.pageinfo.cropbox, offset)
+            trimbox = _offset_rect(page_context.pageinfo.trimbox, offset)
+
+            if swap_axis:
+                cropbox = cropbox[1], cropbox[0], cropbox[3], cropbox[2]
+                trimbox = trimbox[1], trimbox[0], trimbox[3], trimbox[2]
+            page.CropBox = cropbox
+            page.TrimBox = trimbox
+        pdf.save(out_file)
+    return out_file
 
 
 def generate_postscript_stub(context: PdfContext) -> Path:
@@ -822,7 +896,7 @@ def convert_to_pdfa(input_pdf: Path, input_ps_stub: Path, context: PdfContext) -
     # pikepdf can deal with this, but we make the world a better place by
     # stamping them out as soon as possible.
     with pikepdf.open(input_pdf) as pdf_file:
-        if _repair_docinfo_nuls(pdf_file):
+        if repair_docinfo_nuls(pdf_file):
             pdf_file.save(fix_docinfo_file)
         else:
             safe_symlink(input_pdf, fix_docinfo_file)
@@ -845,25 +919,6 @@ def convert_to_pdfa(input_pdf: Path, input_ps_stub: Path, context: PdfContext) -
     return output_file
 
 
-def _repair_docinfo_nuls(pdf):
-    """If the DocumentInfo block contains NUL characters, remove them.
-
-    If the DocumentInfo block is malformed, log an error and continue.
-    """
-    modified = False
-    try:
-        if not isinstance(pdf.docinfo, pikepdf.Dictionary):
-            raise TypeError("DocumentInfo is not a dictionary")
-        for k, v in pdf.docinfo.items():
-            if isinstance(v, str) and b'\x00' in bytes(v):
-                pdf.docinfo[k] = bytes(v).replace(b'\x00', b'')
-                modified = True
-    except TypeError:
-        # TypeError can also be raised if dictionary items are unexpected types
-        log.error("File contains a malformed DocumentInfo block - continuing anyway.")
-    return modified
-
-
 def should_linearize(working_file: Path, context: PdfContext) -> bool:
     """Determine whether the PDF should be linearized.
 
@@ -898,86 +953,6 @@ def get_pdf_save_settings(output_type: str) -> dict[str, Any]:
         )
 
 
-def metadata_fixup(working_file: Path, context: PdfContext) -> Path:
-    """Fix certain metadata fields after Ghostscript PDF/A conversion.
-
-    Also report on metadata in the input file that was not retained during
-    PDF/A conversion.
-    """
-    output_file = context.get_path('metafix.pdf')
-    options = context.options
-
-    def report_on_metadata(missing):
-        if not missing:
-            return
-        if options.output_type.startswith('pdfa'):
-            log.warning(
-                "Some input metadata could not be copied because it is not "
-                "permitted in PDF/A. You may wish to examine the output "
-                "PDF's XMP metadata."
-            )
-            log.debug("The following metadata fields were not copied: %r", missing)
-        else:
-            log.error(
-                "Some input metadata could not be copied."
-                "You may wish to examine the output PDF's XMP metadata."
-            )
-            log.info("The following metadata fields were not copied: %r", missing)
-
-    with pikepdf.open(context.origin) as original, pikepdf.open(working_file) as pdf:
-        docinfo = get_docinfo(original, context)
-        with original.open_metadata(
-            set_pikepdf_as_editor=False, update_docinfo=False, strict=False
-        ) as meta_original, pdf.open_metadata() as meta_pdf:
-            meta_pdf.load_from_docinfo(
-                docinfo, delete_missing=False, raise_failure=False
-            )
-            # If xmp:CreateDate is missing, set it to the modify date to
-            # ensure consistency with Ghostscript.
-            if 'xmp:CreateDate' not in meta_pdf:
-                meta_pdf['xmp:CreateDate'] = meta_pdf.get('xmp:ModifyDate', '')
-            if meta_pdf.get('dc:title') == 'Untitled':
-                # Ghostscript likes to set title to Untitled if omitted from input.
-                # Reverse this, because PDF/A TechNote 0003:Metadata in PDF/A-1
-                # and the XMP Spec do not make this recommendation.
-                if 'dc:title' not in meta_original:
-                    del meta_pdf['dc:title']
-            # If the user explicitly specified an empty string for any of the
-            # following, they should be unset and not reported as missing in
-            # the output pdf. Note that some metadata fields use differing names
-            # between PDF-A and PDF.
-            for meta in [meta_pdf, meta_original]:
-                if options.title == '' and 'dc:title' in meta:
-                    del meta['dc:title']  # PDF-A and PDF
-                if options.author == '':
-                    if 'dc:creator' in meta:
-                        del meta['dc:creator']  # PDF-A (Not xmp:CreatorTool)
-                    if 'pdf:Author' in meta:
-                        del meta['pdf:Author']  # PDF
-                if options.subject == '':
-                    if 'dc:description' in meta:
-                        del meta['dc:description']  # PDF-A
-                    if 'dc:subject' in meta:
-                        del meta['dc:subject']  # PDF
-                if options.keywords == '' and 'pdf:Keywords' in meta:
-                    del meta['pdf:Keywords']  # PDF-A and PDF
-            meta_missing = set(meta_original.keys()) - set(meta_pdf.keys())
-            report_on_metadata(meta_missing)
-
-        optimizing = context.plugin_manager.hook.is_optimization_enabled(
-            context=context
-        )
-        pdf.save(
-            output_file,
-            **get_pdf_save_settings(options.output_type),
-            linearize=(  # Don't linearize if optimize() will be linearizing too
-                not optimizing and should_linearize(working_file, context)
-            ),
-        )
-
-    return output_file
-
-
 def _file_size_ratio(
     input_file: Path, output_file: Path
 ) -> tuple[float | None, float | None]:
@@ -1025,7 +1000,7 @@ def optimize_pdf(
 
 def enumerate_compress_ranges(
     iterable: Iterable[T],
-) -> Iterator[tuple[tuple[int, int], T]]:
+) -> Iterator[tuple[tuple[int, int], T | None]]:
     """Enumerate the ranges of non-empty elements in an iterable.
 
     Compresses consecutive ranges of length 1 into single elements.
@@ -1079,14 +1054,14 @@ def merge_sidecars(txt_files: Iterable[Path | None], context: PdfContext) -> Pat
 
 
 def copy_final(
-    input_file: Path, output_file: str | Path | BinaryIO, _context: PdfContext
+    input_file: Path, output_file: str | Path | BinaryIO, original_file: Path | None
 ) -> None:
     """Copy the final temporary file to the output destination.
 
     Args:
-        input_file (Path): The input file to copy.
+        input_file (Path): The intermediate input file to copy.
         output_file (str | Path | BinaryIO): The output file to copy to.
-        _context (PdfContext): The PDF context.
+        original_file: The original file to copy attributes from.
 
     Returns:
         None
@@ -1105,5 +1080,5 @@ def copy_final(
             # At this point we overwrite the output_file specified by the user
             # use copyfileobj because then we use open() to create the file and
             # get the appropriate umask, ownership, etc.
-            with open(output_file, 'wb') as output_stream:
+            with open(output_file, 'w+b') as output_stream:
                 copyfileobj(input_stream, output_stream)
