@@ -1,17 +1,17 @@
-# © 2020 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+"""Plugin manager using pluggy."""
+
+from __future__ import annotations
 
 import argparse
 import importlib
 import importlib.util
 import pkgutil
 import sys
+from collections.abc import Sequence
 from pathlib import Path
-from typing import List, Sequence, Tuple, Union
 
 import pluggy
 
@@ -33,7 +33,7 @@ class OcrmypdfPluginManager(pluggy.PluginManager):
     def __init__(
         self,
         *args,
-        plugins: List[Union[str, Path]],
+        plugins: Sequence[str | Path],
         builtins: bool = True,
         **kwargs,
     ):
@@ -100,23 +100,30 @@ class OcrmypdfPluginManager(pluggy.PluginManager):
             self.register(module)
 
 
-def get_plugin_manager(plugins: List[Union[str, Path]], builtins=True):
-    pm = OcrmypdfPluginManager(
+def get_plugin_manager(
+    plugins: Sequence[str | Path] | None = None, builtins=True
+) -> OcrmypdfPluginManager:
+    return OcrmypdfPluginManager(
         project_name='ocrmypdf',
-        plugins=plugins,
+        plugins=plugins if plugins is not None else [],
         builtins=builtins,
     )
-    return pm
 
 
 def get_parser_options_plugins(
     args: Sequence[str],
-) -> Tuple[argparse.ArgumentParser, argparse.Namespace, pluggy.PluginManager]:
+) -> tuple[argparse.ArgumentParser, argparse.Namespace, pluggy.PluginManager]:
     pre_options, _unused = plugins_only_parser.parse_known_args(args=args)
     plugin_manager = get_plugin_manager(pre_options.plugins)
 
     parser = get_parser()
+    plugin_manager.hook.initialize(  # pylint: disable=no-member
+        plugin_manager=plugin_manager
+    )
     plugin_manager.hook.add_options(parser=parser)  # pylint: disable=no-member
 
     options = parser.parse_args(args=args)
     return parser, options, plugin_manager
+
+
+__all__ = ['OcrmypdfPluginManager', 'get_plugin_manager', 'get_parser_options_plugins']
