@@ -10,7 +10,7 @@ import multiprocessing
 import os
 import shutil
 import warnings
-from collections.abc import Iterable, Sequence
+from collections.abc import Callable, Iterable, Sequence
 from contextlib import suppress
 from decimal import Decimal
 from io import StringIO
@@ -19,13 +19,13 @@ from pathlib import Path
 from statistics import harmonic_mean
 from typing import (
     Any,
-    Callable,
     Generic,
     TypeVar,
 )
 
 import img2pdf
 import pikepdf
+from deprecation import deprecated
 
 log = logging.getLogger(__name__)
 
@@ -136,6 +136,7 @@ class Resolution(Generic[T]):
         return self._isclose(self.x, other.x) and self._isclose(self.y, other.y)
 
 
+@deprecated(deprecated_in='15.4.0')
 class NeverRaise(Exception):
     """An exception that is never raised."""
 
@@ -267,7 +268,9 @@ def check_pdf(input_file: Path) -> bool:
         return False
     else:
         with pdf:
-            messages = pdf.check()
+            with warnings.catch_warnings():
+                warnings.filterwarnings('ignore', message=r'pikepdf.*JBIG2.*')
+                messages = pdf.check()
             success = True
             for msg in messages:
                 if 'error' in msg.lower():
@@ -319,10 +322,30 @@ def remove_all_log_handlers(logger: logging.Logger) -> None:
 
 
 def pikepdf_enable_mmap() -> None:
-    """Enable pikepdf mmap."""
+    """Enable pikepdf memory mapping."""
     try:
-        if pikepdf._core.set_access_default_mmap(True):
-            log.debug("pikepdf mmap enabled")
+        pikepdf._core.set_access_default_mmap(True)
+        log.debug(
+            "pikepdf mmap "
+            + (
+                'enabled'
+                if pikepdf._core.get_access_default_mmap()  # type: ignore[attr-defined]
+                else 'disabled'
+            )
+        )
     except AttributeError:
         log.debug("pikepdf mmap not available")
-    log.debug("pikepdf mmap disabled")
+
+
+def running_in_docker() -> bool:
+    """Returns True if we seem to be running in a Docker container."""
+    return Path('/.dockerenv').exists()
+
+
+def running_in_snap() -> bool:
+    """Returns True if we seem to be running in a Snap container."""
+    try:
+        cgroup_text = Path('/proc/self/cgroup').read_text()
+        return 'snap.ocrmypdf' in cgroup_text
+    except FileNotFoundError:
+        return False
