@@ -4,11 +4,11 @@
 from __future__ import annotations
 
 import os
-from subprocess import PIPE, run
+from subprocess import run
 
 import pytest
 
-from .conftest import running_in_docker
+from ocrmypdf.helpers import running_in_docker
 
 pytestmark = pytest.mark.skipif(
     running_in_docker(),
