@@ -75,7 +75,7 @@ def test_img2pdf_fails(resources, no_outpdf):
 @pytest.mark.xfail(reason="remove background disabled")
 def test_jpeg_in_jpeg_out(resources, outpdf):
     check_ocrmypdf(
-        resources / 'congress.jpg',
+        resources / 'baiona_color.jpg',
         outpdf,
         '--image-dpi',
         '100',
