@@ -15,13 +15,14 @@ from pikepdf.models.metadata import decode_pdf_date
 
 from ocrmypdf._exec import ghostscript
 from ocrmypdf._jobcontext import PdfContext
-from ocrmypdf._pipeline import convert_to_pdfa, metadata_fixup
+from ocrmypdf._metadata import metadata_fixup
+from ocrmypdf._pipeline import convert_to_pdfa
 from ocrmypdf._plugin_manager import get_parser_options_plugins, get_plugin_manager
 from ocrmypdf.exceptions import ExitCode
 from ocrmypdf.pdfa import file_claims_pdfa, generate_pdfa_ps
 from ocrmypdf.pdfinfo import PdfInfo
 
-from .conftest import check_ocrmypdf, run_ocrmypdf
+from .conftest import check_ocrmypdf, run_ocrmypdf, run_ocrmypdf_api
 
 
 @pytest.mark.parametrize("output_type", ['pdfa', 'pdf'])
@@ -34,9 +35,10 @@ def test_preserve_docinfo(output_type, resources, outpdf):
         '--plugin',
         'tests/plugins/tesseract_noop.py',
     )
-    with pikepdf.open(resources / 'graph.pdf') as pdf_before, pikepdf.open(
-        output
-    ) as pdf_after:
+    with (
+        pikepdf.open(resources / 'graph.pdf') as pdf_before,
+        pikepdf.open(output) as pdf_after,
+    ):
         for key in ('/Title', '/Author'):
             assert pdf_before.docinfo[key] == pdf_after.docinfo[key]
         pdfa_info = file_claims_pdfa(str(output))
@@ -44,12 +46,12 @@ def test_preserve_docinfo(output_type, resources, outpdf):
 
 
 @pytest.mark.parametrize("output_type", ['pdfa', 'pdf'])
-def test_override_metadata(output_type, resources, outpdf):
+def test_override_metadata(output_type, resources, outpdf, caplog):
     input_file = resources / 'c02-22.pdf'
     german = 'Du siehst den Wald vor lauter Bäumen nicht.'
     chinese = '孔子'
 
-    p = run_ocrmypdf(
+    exitcode = run_ocrmypdf_api(
         input_file,
         outpdf,
         '--title',
@@ -62,7 +64,7 @@ def test_override_metadata(output_type, resources, outpdf):
         'tests/plugins/tesseract_noop.py',
     )
 
-    assert p.returncode == ExitCode.ok, p.stderr
+    assert exitcode == ExitCode.ok, caplog.text
 
     with pikepdf.open(input_file) as before, pikepdf.open(outpdf) as after:
         assert after.docinfo.Title == german, after.docinfo
@@ -79,7 +81,7 @@ def test_override_metadata(output_type, resources, outpdf):
 
 @pytest.mark.parametrize('output_type', ['pdfa', 'pdf', 'pdfa-1', 'pdfa-2', 'pdfa-3'])
 @pytest.mark.parametrize('field', ['title', 'author', 'subject', 'keywords'])
-def test_unset_metadata(output_type, field, resources, outpdf):
+def test_unset_metadata(output_type, field, resources, outpdf, caplog):
     input_file = resources / 'meta.pdf'
 
     # magic strings contained in the input pdf metadata
@@ -90,7 +92,7 @@ def test_unset_metadata(output_type, field, resources, outpdf):
         'keywords': b's9EeALwUg7urA7fnnhm5EtUyC54sW2WPUzqh',
     }
 
-    p = run_ocrmypdf(
+    exitcode = run_ocrmypdf_api(
         input_file,
         outpdf,
         f'--{field}',
@@ -101,7 +103,7 @@ def test_unset_metadata(output_type, field, resources, outpdf):
         'tests/plugins/tesseract_noop.py',
     )
 
-    assert p.returncode == ExitCode.ok, p.stderr
+    assert exitcode == ExitCode.ok, caplog.text
 
     # We mainly want to ensure that when '' is passed, the corresponding
     # metadata is unset in the output pdf. Since metedata is not compressed,
@@ -332,7 +334,9 @@ def test_metadata_fixup_warning(resources, outdir, caplog):
     context = PdfContext(
         options, outdir, outdir / 'graph.pdf', None, get_plugin_manager([])
     )
-    metadata_fixup(working_file=outdir / 'graph.pdf', context=context)
+    metadata_fixup(
+        working_file=outdir / 'graph.pdf', context=context, pdf_save_settings={}
+    )
     for record in caplog.records:
         assert record.levelname != 'WARNING', "Unexpected warning"
 
@@ -345,7 +349,9 @@ def test_metadata_fixup_warning(resources, outdir, caplog):
     context = PdfContext(
         options, outdir, outdir / 'graph_mod.pdf', None, get_plugin_manager([])
     )
-    metadata_fixup(working_file=outdir / 'graph.pdf', context=context)
+    metadata_fixup(
+        working_file=outdir / 'graph.pdf', context=context, pdf_save_settings={}
+    )
     assert any(record.levelname == 'WARNING' for record in caplog.records)
 
 
